/*
 * ICalculator.java
 *
 * Created on December 4, 2002, 11:49 PM
 */

package Calculator;

import java.util.Iterator;

/** The interface to a calculator model.
 * The calculator can evaluate infix expressions involving integer
 * values, and the operators +, -, *, and / (integer division).  The expressions
 * can have parentheses nested arbitrarily deep.  The same precedence and associativity
 * rules apply as in Java.  The calculator doesn't have to handle unary minus or
 * the mod operator.
 *
 * The calculator responds to an "evaluate" command by processing the list of tokens
 * supplied and computing the overall value of the expression.  The tokens are
 * strings (more details below) and the default number base for integer tokens is 10.
 *
 * Implementors of the model should supply a constructor with no arguments.  After
 * completion of the constructor, the model should be ready to accept commands.
 *
 * The command "terminate" means that no more demands will be placed on the model, 
 * so it is free to cleanup or terminate.
 *
 * In addition to any exceptions listed below, the methods may throw an exception or 
 * otherwise signal an error if called when the model has already been terminated.
 *
 */
public interface ICalculatorModel {
    
    /** Set the next expression to be evaluated. 
     * The previous expression is forgotten.
     * Note: it is NOT an error to set a new expression when the old one's value
     * has never been called for.
     * 
      @param expressionTokens An iterator to a list of the tokens in the expression
     * in their original, left-to-right, infix order.  Each token is a string.
     * For example, the expression 1 + (300 - 17) would correspond to a token 
     * list of: "1" "+" "(" "300" "-" "17" ")".
     * Pre: The token list is non-empty
     * Post: If the token list was valid (in all respects relative to the model's
     * ability to evaluate it), then the expression has been saved within the model.
     * @return -1 if the token list was valid.  If not valid, the return value is
     * a list position where an error was detected.  By convention, this should
     * be the position of the first error, but that need not be guaranteed.
     */
    public int setExpression(Iterator expressionTokens);
    
    /** Answer, what is the value of the latest expression?
     * The current expression is the last valid expression received.
     * Pre: a valid expressed is present.
     * Post: None of the tokens have been modified.
     * @return the integer value of the last valid expression.
     * @throws RuntimeException If no valid expression has yet been received, or the latest expression
     * was invalid, the method should throw a runtime error, with a message
     * (in the exception) describing the reason for the exception.
     */
    public int evaluateExpression();
    
    /** Answer, what is the current expression?
     * @return a string with all the tokens of the original expression, in the
     * usual left-to-right infix notation.  Note that the current expression might
     * have been invalid, but this method will still return it.
     * @throws RuntimeException is no expression has yet been set.
     */
    public String currentExpression();
    
    /** Set the number base which will be assumed for integer tokens in all subsequent new
     * expressions.
     *
     * Models are free to accept or reject any base except 10, which must always
     * be accepted.
     * @return true if the model accepts the change to the desired number base.
     * @return false if the model does not accept the change.
     * NOTE: This method was intended to be in the interface, but was
     *  accidentally commented out.  It will be left commented out.
    public boolean setNumberBase(int radix);
    
    /** Answer, what is the value of the current expression?
     * @return the (correct) value of the current expression, if there is one, 
        as a string with no leading
     *  or trailing whitespace and no leading zeros.  If the current expression
     * is invalid, return an empty string (not a null reference).
     * @throws RuntimeException is no expression has yet been set.
     */
    public String currentValue();
    
    /** Signal that the model will not be called upon again.
     */
    public void terminate();

// end ICalculator
}
